// $Id: checksyntax-vserver.hc,v 1.2 2004/01/17 07:38:14 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <vserver.h>
#include <stdio.h>

static bool
checkSyntax_vserverName(char const *str, char const **reason)
{
  assert(str!=0);
  assert(reason!=0);
  
  if (strlen(str)>32) {
    *reason = "vserver-name too long";
    return false;
  }

  if (strlen(str)==0) {
    *reason = "vserver-name too short";
    return false;
  }
  
  if (!isalpha(*str)) {
    *reason = "vserver-name does not begin with a character";
    return false;
  }

  for (++str; *str!='\0'; ++str) {
    if (!isalpha(*str) &&
	!isdigit(*str) &&
	*str!='-' && *str!='_' && *str!='.') {
      *reason = "Invalid character in vserver-name";
      return false;
    }
  }

  return true;
}

static bool
checkSyntax_vserverInternal(struct Parameters *self,
			    char const *str, char const **reason)
{
  if (strlen(str)>128) {
    *reason = "vserver-name too long";
    return false;
  }

  vcCfgStyle		style = vc_getVserverCfgStyle(str);

  self->vserver_name = vc_getVserverName(str, style);
  self->vserver_vdir = vc_getVserverVdir(str, style, false);

  if (self->vserver_name==0 ||
      self->vserver_vdir==0) {
    *reason = "Invalid/non existing vserver";
    return false;
  }

  return checkSyntax_vserverName(self->vserver_name, reason);
}

static bool
checkSyntax_vserver(struct Parameters *self,
		    char const *str, struct DjinniProcess const *proc,
		    char const **reason)
{
  int		fd = open(".", O_RDONLY);

  if (fd==-1) {
    *reason = "Internal error: failed to bookmark current dir";
    return false;
  }
  
  if (!DjinniProcess_chrootVserver(proc)) {
    perror("DjinniProcess_chrootVserver()");
    *reason = "Internal error: failed to enter current vserver";
    close(fd);
    return false;
  }
  
  int		res = checkSyntax_vserverInternal(self, str, reason);

  if (!DjinniProcess_chrootCurrent(proc) ||
      fchdir(fd)==-1 || close(fd)==-1) {
    perror("DjinniProcess_chrootCurrent()");
    // We do not have another chance; when this happens, the state is
    // completely screwed up
    exit(1);
  }
  
  return res;
}
