// $Id: djinniprocess.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "djinniprocess.h"
#include "djinniprocess-internal.h"
#include "wrappers.h"
#include "util-io.h"
#include "util.h"

#include <sys/wait.h>

void
DjinniProcess_doQuit(struct DjinniProcess *self)
{
  self->is_alive = false;
}

void
DjinniProcess_detach(struct DjinniProcess *self)
{
  XXclose(&self->fd_parent);
  XXclose(&self->fd_cmd_vserver);
  XXclose(&self->fd_cmd_master);
  XXclose(&self->fd_heartbeat_vserver);
  XXclose(&self->fd_heartbeat_children[1]);
  
  XXclose(&self->dirfd_root);
  XXclose(&self->dirfd_state);
  XXclose(&self->dirfd_vserver);
}

void
DjinniProcess_delete(struct DjinniProcess *self)
{
  static char const * const	RM_FILES[] = {
    ".cfg", ".ctx", ".cmd", ".root", ".vdir", ".trusted", 0 };
  char const * const *		file;
  bool				good_dirstate = false;

  Xclose(self->fd_heartbeat_children[1]);

  good_dirstate = (self->dirfd_root!=-1 &&
		   fchdir(self->dirfd_root)!=-1 &&
		   chroot(".")!=-1 &&
		   self->dirfd_state!=-1 &&
		   fchdir(self->dirfd_state)!=-1);

  if (good_dirstate) {
    for (file=RM_FILES; *file!=0; ++file)
      unlink(*file);
  }
  else {
    WRITE_MSG(2, "Could not restore original directory state; will skip cleanup of state-direcotry therefore.\n");
  }
  
  // TODO: logging
  while (TEMP_FAILURE_RETRY(wait4(-1,0,0,0))!=-1) {}
  if (errno!=ECHILD)
    perror("wait4()");

  if (!good_dirstate ||
      (self->parent!=0 && rmdir(self->state_dir))==-1) {
    WRITE_MSG(2, "Can not unlink statedir '");
    WRITE_STR(2, self->state_dir);
    WRITE_MSG(2, "'\n");
  }
  free(const_cast(char *)(self->state_dir));
  free(self);
}
