// $Id: wrappers.h,v 1.3 2003/11/07 00:43:21 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifndef H_VSERVER_DJINNI_SRC_WRAPPERS_H
#define H_VSERVER_DJINNI_SRC_WRAPPERS_H

#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>

#define WRAPPER_DECL	UNUSED ALWAYSINLINE

inline static UNUSED void 
FatalErrnoError(bool condition, char const msg[]) /*@*/
{
  if (!condition)       return;

#ifdef DO_WRAPPER_LOGGING
  extern void Logging_timestamp(int);
  Logging_timestamp(2);
#endif
  
  perror(msg);
  

  extern int	wrapper_exit_code;
  exit(wrapper_exit_code);
}

inline static UNUSED void
Eclose(int s)
{
  FatalErrnoError(close(s)==-1, "close()");
}

inline static UNUSED size_t
Eread(int fd, void *ptr, size_t len)
{
  size_t	res = read(fd, ptr, len);
  FatalErrnoError((ssize_t)(res)==-1, "read()");

  return res;
}

inline static UNUSED size_t
Ewrite(int fd, void const *ptr, size_t len)
{
  size_t	res = write(fd, ptr, len);
  FatalErrnoError((ssize_t)(res)==-1, "write()");

  return res;
}

inline static WRAPPER_DECL void
Epipe(int modus[2])
{
  register int		res = pipe(modus);
  FatalErrnoError(res==-1, "pipe()");
}

inline static WRAPPER_DECL int
Edup2(int oldfd, int newfd)
{
  register int		res = dup2(oldfd, newfd);
  FatalErrnoError(res==-1, "dup2()");

  return res;
}


inline static WRAPPER_DECL void
Esocketpair(int d, int type, int protocol, int *sv)
{
  register int		res = socketpair(d,type,protocol,sv);
  FatalErrnoError(res==-1, "socketpair()");
}

inline static UNUSED pid_t
Efork()
{
  pid_t		res = fork();
  FatalErrnoError(res==-1, "fork()");

  return res;
}

inline static UNUSED void
Esetsockopt(int s, int level, int optname, const void *optval, socklen_t optlen)
{
  FatalErrnoError(setsockopt(s, level, optname, optval, optlen)==-1,
		  "setsockopt()");
}

inline static UNUSED void *
Emalloc(size_t size)
{
  register void               *res = malloc(size);
  FatalErrnoError(res==0 && size!=0, "malloc()");
  return res;
}

inline static UNUSED int
Esocket(int domain, int type, int protocol)
{
  register int		res = socket(domain, type, protocol);
  FatalErrnoError(res==-1, "socket()");

  return res;
}

inline static UNUSED void
Econnect(int fd, void const *addr, socklen_t len)
{
  register int		res = connect(fd, addr, len);
  FatalErrnoError(res==-1, "connect()");
}

inline static WRAPPER_DECL int
Eaccept(int s, void *addr, socklen_t *addrlen)
{
  register int		res = accept(s,addr,addrlen);
  FatalErrnoError(res==-1, "accept()");

  return res;
}


inline static WRAPPER_DECL int
Eopen(char const *fname, int flags, mode_t mode)
{
  int	res = open(fname, flags, mode);
  FatalErrnoError(res==-1, "open()");

  return res;
}

inline static WRAPPER_DECL void
Esymlink(const char *oldpath, const char *newpath)
{
  FatalErrnoError(symlink(oldpath, newpath)==-1, "symlink()");
}

inline static WRAPPER_DECL void
Ebind(int sockfd, void *my_addr, socklen_t addrlen)
{
  FatalErrnoError(bind(sockfd, my_addr, addrlen)==-1, "bind()");
}

inline static WRAPPER_DECL void
Eatexit(void (*function)(void))
{
  if (atexit(function)==-1) {
    extern int	wrapper_exit_code;
    
    (void)write(2, "atexit(): failed to register handler\n", 37);
    exit(wrapper_exit_code);
  }
}

inline static WRAPPER_DECL void
Emkdir(const char *pathname, mode_t mode)
{
  FatalErrnoError(mkdir(pathname,mode)==-1, "mkdir()");
}

inline static WRAPPER_DECL void
Echdir(char const path[])
{
  FatalErrnoError(chdir(path)==-1, "chdir()");
}

inline static WRAPPER_DECL pid_t
Esetsid()
{
  register pid_t const	res = setsid();
  FatalErrnoError(res==-1, "setsid()");

  return res;
}

inline static WRAPPER_DECL int
Eselect(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
	struct timeval *timeout)
{
  register int                  res;
  
  res = select(n, readfds, writefds, exceptfds, timeout);
  FatalErrnoError(res==-1, "select()");

  return res;
}

inline static WRAPPER_DECL void
Echroot(char const path[])
{
  FatalErrnoError(chroot(path)==-1, "chroot()");
}


inline static WRAPPER_DECL void
Efchdir(int fd)
{
  FatalErrnoError(fchdir(fd)==-1, "fchdir()");
}

inline static WRAPPER_DECL void
Elisten(int fd, int backlog)
{
  FatalErrnoError(listen(fd,backlog)==-1, "listen()");
}

inline static WRAPPER_DECL off_t
Elseek(int fildes, off_t offset, int whence)
{
  off_t		res = lseek(fildes, offset, whence);
  FatalErrnoError(res==(off_t)-1, "lseek()");
  return res;
}

inline static WRAPPER_DECL void
Eexecv(char const *path, char *argv[])
{
  FatalErrnoError(execv(path,argv)==-1, "execv()");
}

inline static WRAPPER_DECL void
Esetenv(char const *name, char const *value, int overwrite)
{
  FatalErrnoError(setenv(name,value,overwrite)==-1, "setenv()");
}

inline static WRAPPER_DECL void
Efstat(int filedes, struct stat *buf)
{
  FatalErrnoError(fstat(filedes, buf)==-1, "fstat()");
}

inline static WRAPPER_DECL void
Emknod(const char *pathname, mode_t mode, dev_t dev)
{
  FatalErrnoError(mknod(pathname, mode, dev)==-1, "mknod()");
}

inline static WRAPPER_DECL void
Eunsetenv(char const *name)
{
#ifdef __dietlibc__
  unsetenv(name);
#else
  FatalErrnoError(unsetenv(name)==-1, "unsetenv()");
#endif
}

#endif	//  H_VSERVER_DJINNI_SRC_WRAPPERS_H
